%> 	\authors John-Olof Nilsson, Isaac Skog
%>	\copyright Copyright (c) 2011 OpenShoe, 
%> Opensource License: Creative Commons Attribution 4.0 International 
%> Pulic License

% Clear workspac
clear all, clc, close all

% Open serial port
com = serial('COM10', 'BaudRate',460800,'InputBufferSize',10000,'ByteOrder','bigEndian');
fopen(com);
pause(1);

% Figure for plotting
figure(1)
hold on
%grid on
title('Real-time position plot');
xlabel('x [m]');
ylabel('y [m]');
% Add pushbutton such that logging can be aborted
abort_flag = 0;
uicontrol('style','push','string','Stop','callback','abort_flag=1;');
drawnow
pause(0.1)

nr_steps = 60;
figure(1)
grid on
lines = zeros(nr_steps,1);
for i=1:nr_steps
    lines(i) = line('XData',[],'YData',[]);
end

settings();
global simdata;
x_sw = zeros(4,1);
x_sw(1:3) = simdata.init_pos;
x_sw(4) = simdata.init_heading;
x_sw_old = x_sw;
P_sw = zeros(4,4);
P_sw(1:3,1:3)=diag(simdata.sigma_initial_pos.^2);
P_sw(4,4)=simdata.sigma_initial_att(3)^2;
P_sw_old = P_sw;

file = fopen('swdr_data.txt', 'w');

% Reset stepwise deadreckoning INS
fwrite(com,[20 0 20],'uint8');
fread(com,4,'uint8')

package_number_old = nan;
while abort_flag==0
     if(com.BytesAvailable>=64)
        header = fread(com,2,'uint8'); % Header + number + Payload size (-) 4, (+)2
        payload = fread(com,14,'float');
        step_counter = fread(com,1,'uint16'); % Step counter
        fread(com,1,'uint16'); % Checksum
%         package_number1 = header(2);
%         package_number2 = header(3);
%         ack = [1 package_number1 package_number2 (1+package_number1+package_number2-mod(1+package_number1+package_number2,256))/256 mod(1+package_number1+package_number2,256)];
%         fwrite(com,ack,'uint8');
%         package_number = package_number1*256 + package_number2;
%         if package_number_old ~= package_number
%             fprintf(file,'%i %i %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f
%             %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f
%             %12.9f\n',step_counter,package_number,payload);% for wireless
             fprintf(file,'%i %i %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f %12.9f\n',step_counter,payload);% for USB
            dx = payload(1:4);
            dP = Pvec2Pmat(payload(5:14));
            pdef=find(eig(dP)<=0);
            if isempty(pdef)
                chol(dP,'lower');
            end
            [x_sw P_sw] = stepwise_dr_tu(x_sw,P_sw,dx,dP);
            figure(1)
            plot([x_sw(1) x_sw_old(1)],[x_sw(2) x_sw_old(2)],'b');
            axis equal;
             drawnow;
             x_sw_old = x_sw;
             P_sw_old = P_sw;
%          else
%             disp('package sent twice');
%         end
%         package_number_old = package_number;
%         disp(package_number_old);
     end
end

% Stop output
fwrite(com,[39 0 39],'uint8'); % Processing off
fwrite(com,[33 0 33],'uint8'); % all output off

pause(1);
if com.BytesAvailable>0
    fread(com,com.BytesAvailable,'uint8')
end

disp(x_sw)

fclose(com);
fclose(file);



